# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/hdmap_lib/proto/geometry.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from proto.hdmap_lib.proto import id_pb2 as proto_dot_hdmap__lib_dot_proto_dot_id__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='proto/hdmap_lib/proto/geometry.proto',
  package='hdmap_lib',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n$proto/hdmap_lib/proto/geometry.proto\x12\thdmap_lib\x1a\x1eproto/hdmap_lib/proto/id.proto\"-\n\x07Point3D\x12\t\n\x01x\x18\x01 \x01(\x01\x12\t\n\x01y\x18\x02 \x01(\x01\x12\x0c\n\x01z\x18\x03 \x01(\x01:\x01\x30\"\xd9\x02\n\x0eLineStringData\x12\x19\n\x02id\x18\x01 \x01(\x0b\x32\r.hdmap_lib.Id\x12.\n\x05\x63olor\x18\x02 \x01(\x0e\x32\x1f.hdmap_lib.LineStringData.Color\x12,\n\x04type\x18\x03 \x01(\x0e\x32\x1e.hdmap_lib.LineStringData.Type\x12\"\n\x06points\x18\x04 \x03(\x0b\x32\x12.hdmap_lib.Point3D\x12\x0e\n\x06length\x18\x05 \x01(\x01\"\x1e\n\x05\x43olor\x12\t\n\x05WHITE\x10\x01\x12\n\n\x06YELLOW\x10\x02\"z\n\x04Type\x12\x08\n\x04NONE\x10\x01\x12\t\n\x05SOLID\x10\x02\x12\n\n\x06\x42ROKEN\x10\x03\x12\x10\n\x0c\x44OUBLE_SOLID\x10\x04\x12\x11\n\rDOUBLE_BROKEN\x10\x05\x12\x10\n\x0c\x42ROKEN_SOLID\x10\x06\x12\x10\n\x0cSOLID_BROKEN\x10\x07\x12\x08\n\x04\x43URB\x10\x08\"B\n\nLineString\x12\x19\n\x02id\x18\x01 \x01(\x0b\x32\r.hdmap_lib.Id\x12\x19\n\nis_reverse\x18\x02 \x01(\x08:\x05\x66\x61lse\"-\n\x05\x43urve\x12$\n\x05lines\x18\x01 \x03(\x0b\x32\x15.hdmap_lib.LineString\"/\n\x07Polygon\x12$\n\x05lines\x18\x01 \x03(\x0b\x32\x15.hdmap_lib.LineString'
  ,
  dependencies=[proto_dot_hdmap__lib_dot_proto_dot_id__pb2.DESCRIPTOR,])



_LINESTRINGDATA_COLOR = _descriptor.EnumDescriptor(
  name='Color',
  full_name='hdmap_lib.LineStringData.Color',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='WHITE', index=0, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='YELLOW', index=1, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=322,
  serialized_end=352,
)
_sym_db.RegisterEnumDescriptor(_LINESTRINGDATA_COLOR)

_LINESTRINGDATA_TYPE = _descriptor.EnumDescriptor(
  name='Type',
  full_name='hdmap_lib.LineStringData.Type',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NONE', index=0, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SOLID', index=1, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BROKEN', index=2, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DOUBLE_SOLID', index=3, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DOUBLE_BROKEN', index=4, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BROKEN_SOLID', index=5, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SOLID_BROKEN', index=6, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CURB', index=7, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=354,
  serialized_end=476,
)
_sym_db.RegisterEnumDescriptor(_LINESTRINGDATA_TYPE)


_POINT3D = _descriptor.Descriptor(
  name='Point3D',
  full_name='hdmap_lib.Point3D',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='hdmap_lib.Point3D.x', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='y', full_name='hdmap_lib.Point3D.y', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='z', full_name='hdmap_lib.Point3D.z', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=True, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=83,
  serialized_end=128,
)


_LINESTRINGDATA = _descriptor.Descriptor(
  name='LineStringData',
  full_name='hdmap_lib.LineStringData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='hdmap_lib.LineStringData.id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='color', full_name='hdmap_lib.LineStringData.color', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='hdmap_lib.LineStringData.type', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='points', full_name='hdmap_lib.LineStringData.points', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='length', full_name='hdmap_lib.LineStringData.length', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _LINESTRINGDATA_COLOR,
    _LINESTRINGDATA_TYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=131,
  serialized_end=476,
)


_LINESTRING = _descriptor.Descriptor(
  name='LineString',
  full_name='hdmap_lib.LineString',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='hdmap_lib.LineString.id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_reverse', full_name='hdmap_lib.LineString.is_reverse', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=478,
  serialized_end=544,
)


_CURVE = _descriptor.Descriptor(
  name='Curve',
  full_name='hdmap_lib.Curve',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='lines', full_name='hdmap_lib.Curve.lines', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=546,
  serialized_end=591,
)


_POLYGON = _descriptor.Descriptor(
  name='Polygon',
  full_name='hdmap_lib.Polygon',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='lines', full_name='hdmap_lib.Polygon.lines', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=593,
  serialized_end=640,
)

_LINESTRINGDATA.fields_by_name['id'].message_type = proto_dot_hdmap__lib_dot_proto_dot_id__pb2._ID
_LINESTRINGDATA.fields_by_name['color'].enum_type = _LINESTRINGDATA_COLOR
_LINESTRINGDATA.fields_by_name['type'].enum_type = _LINESTRINGDATA_TYPE
_LINESTRINGDATA.fields_by_name['points'].message_type = _POINT3D
_LINESTRINGDATA_COLOR.containing_type = _LINESTRINGDATA
_LINESTRINGDATA_TYPE.containing_type = _LINESTRINGDATA
_LINESTRING.fields_by_name['id'].message_type = proto_dot_hdmap__lib_dot_proto_dot_id__pb2._ID
_CURVE.fields_by_name['lines'].message_type = _LINESTRING
_POLYGON.fields_by_name['lines'].message_type = _LINESTRING
DESCRIPTOR.message_types_by_name['Point3D'] = _POINT3D
DESCRIPTOR.message_types_by_name['LineStringData'] = _LINESTRINGDATA
DESCRIPTOR.message_types_by_name['LineString'] = _LINESTRING
DESCRIPTOR.message_types_by_name['Curve'] = _CURVE
DESCRIPTOR.message_types_by_name['Polygon'] = _POLYGON
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Point3D = _reflection.GeneratedProtocolMessageType('Point3D', (_message.Message,), {
  'DESCRIPTOR' : _POINT3D,
  '__module__' : 'proto.hdmap_lib.proto.geometry_pb2'
  # @@protoc_insertion_point(class_scope:hdmap_lib.Point3D)
  })
_sym_db.RegisterMessage(Point3D)

LineStringData = _reflection.GeneratedProtocolMessageType('LineStringData', (_message.Message,), {
  'DESCRIPTOR' : _LINESTRINGDATA,
  '__module__' : 'proto.hdmap_lib.proto.geometry_pb2'
  # @@protoc_insertion_point(class_scope:hdmap_lib.LineStringData)
  })
_sym_db.RegisterMessage(LineStringData)

LineString = _reflection.GeneratedProtocolMessageType('LineString', (_message.Message,), {
  'DESCRIPTOR' : _LINESTRING,
  '__module__' : 'proto.hdmap_lib.proto.geometry_pb2'
  # @@protoc_insertion_point(class_scope:hdmap_lib.LineString)
  })
_sym_db.RegisterMessage(LineString)

Curve = _reflection.GeneratedProtocolMessageType('Curve', (_message.Message,), {
  'DESCRIPTOR' : _CURVE,
  '__module__' : 'proto.hdmap_lib.proto.geometry_pb2'
  # @@protoc_insertion_point(class_scope:hdmap_lib.Curve)
  })
_sym_db.RegisterMessage(Curve)

Polygon = _reflection.GeneratedProtocolMessageType('Polygon', (_message.Message,), {
  'DESCRIPTOR' : _POLYGON,
  '__module__' : 'proto.hdmap_lib.proto.geometry_pb2'
  # @@protoc_insertion_point(class_scope:hdmap_lib.Polygon)
  })
_sym_db.RegisterMessage(Polygon)


# @@protoc_insertion_point(module_scope)



import os
from typing import List, Tuple

import proto.grpc.hdmapimpl_pb2 as hdmapimpl_pb2
from config.config import ConfigManager
from externals import libhdmap
from externals.libhdmap import *
from pyhdmap.utils.vessel import *
from pyhdmap.utils.singleton import Singleton
from pyhdmap.utils.exception import except_output

current_file_path = os.path.abspath(__file__)
config_path = os.path.dirname(os.path.dirname(current_file_path))
ConfigManager().init_config(config_path)
runtime_flags = ConfigManager()._get_runtime_flags()
libhdmap.SetRuntimeFlags(runtime_flags)

@Singleton
class LocalManager:
  def __init__(self) -> None:
    self._map_bin_path = ""
    self._business_scene = ""
    self._hdmap = HDMap()
    
    
  def set_bin_path_and_scene(self, bin_path, scene):
    self._map_bin_path = bin_path
    self._business_scene = scene
    
  def LoadMap(self, bridge_mode = {}):
    hdmap_bridge_modes = []
    for bridge_name, mode in bridge_mode.items():
      hdmap_bridge_mode = BridgeMode()
      hdmap_bridge_mode.bridge_name = bridge_name
      hdmap_bridge_mode.mode = mode
      hdmap_bridge_modes.append(hdmap_bridge_mode)
    res = self._hdmap.LoadMapWithMode(self._map_bin_path, self._business_scene, hdmap_bridge_modes)
    return res
  
  ## basic method    
  @except_output
  def Reload(self, vessels:List[Vessel]):
    vessel_status = build_vessel_status(vessels)
    return self._hdmap.Reload(vessel_status.SerializeToString())
  
  @except_output
  def GetMap(self):
    return self._hdmap.GetMap()
  
  @except_output
  def GetLaneById(self, id:int):
    return self._hdmap.GetLaneById(IdInfo(id))
  
  @except_output
  def GetRoadById(self, id:int):
    return self._hdmap.GetRoadById(IdInfo(id))
    
  @except_output
  def GetJunctionById(self, id:int):
    return self._hdmap.GetJunctionById(IdInfo(id))
  
  @except_output
  def GetObjectById(self, id:int):
    return self._hdmap.GetObjectById(IdInfo(id))  
    
  @except_output
  def GetCrosswalkById(self, id:int):
    return self._hdmap.GetCrosswalkById(IdInfo(id))

  @except_output
  def GetParkingSpaceById(self, id:int):
    return self._hdmap.GetParkingSpaceById(IdInfo(id))
  
  @except_output
  def GetPoiById(self, id:int):
    return self._hdmap.GetPoiById(IdInfo(id))
  
  @except_output
  def GetPullOverRegionById(self, id:int):
    return self._hdmap.GetPullOverRegionById(IdInfo(id))
  
  @except_output
  def GetSignalById(self, id:int):
    return self._hdmap.GetSignalById(IdInfo(id))
  
  @except_output
  def GetStopLineById(self, id:int):
    return self._hdmap.GetStopLineById(IdInfo(id))
  
  @except_output
  def GetLanes(self, point, distance):
    return self._hdmap.GetLanes(Vec2d(point.x(), point.y()), distance)
    
  @except_output
  def GetJunctions(self, point, distance):
    return self._hdmap.GetJunctions(Vec2d(point.x(), point.y()), distance)
    
  @except_output
  def GetObjects(self, point, distance):
    return self._hdmap.GetObjects(Vec2d(point.x(), point.y()), distance)
    
  @except_output
  def GetCrosswalks(self, point, distance):
    return self._hdmap.GetCrosswalks(Vec2d(point.x(), point.y()), distance)
    
  @except_output
  def GetParkingSpaces(self, point, distance):
    return self._hdmap.GetParkingSpaces(Vec2d(point.x(), point.y()), distance)


from pyhdmap.remote.impl_cache import ImplCache
from pyhdmap.remote.hdmap_impl.lane_impl import LaneImpl
from pyhdmap.remote.hdmap_impl.road_impl import RoadImpl, LaneSectionImpl
from pyhdmap.remote.hdmap_impl.junction_impl import JunctionImpl
from pyhdmap.remote.hdmap_impl.object_impl import ObjectImpl
from pyhdmap.remote.hdmap_impl.crosswalk_impl import CrosswalkImpl
from pyhdmap.remote.hdmap_impl.parking_space_impl import ParkingSpaceImpl, LanePoint
from pyhdmap.remote.hdmap_impl.poi_impl import PoiImpl
from pyhdmap.remote.hdmap_impl.pull_over_region_impl import PullOverRegionImpl
from pyhdmap.remote.hdmap_impl.signal_impl import SignalImpl
from pyhdmap.remote.hdmap_impl.stop_line_impl import StopLineImpl
from pyhdmap.remote.hdmap_impl.lane_link_impl import LaneLinkImpl
from pyhdmap.remote.hdmap_impl.relation_impl import RelationImpl
from pyhdmap.remote.hdmap_impl.attribute_impl import AttributesImpl
from externals.libhdmap import *

impl_cache = ImplCache()

def ToInfo(impls, type):
  target_impls = []
  info_list = []
  if isinstance(impls, list) == False:
    target_impls.append(impls)
  else:
    target_impls = impls
    
  if type == "lane":
    info_list.extend([LaneInfo(impl) for impl in target_impls])
  elif type == "lane_section":
    info_list.extend([LaneSectionInfo(impl) for impl in target_impls])
  
  return info_list
class LaneInfo:
  def __init__(self, lane_impl:LaneImpl) -> None:
    self._impl = lane_impl
    
  def id(self):
    return self._impl.id()
  
  def type(self):
    return self._impl.type()
  
  def turn_type(self):
    return self._impl.turn_type()
  
  def max_speed(self):
    return self._impl.max_speed()
  
  def min_speed(self):
    return self._impl.min_speed()
  
  def junction_id(self):
    return self._impl.junction_id()
  
  def IsInJunction(self):
    return self._impl.IsInJunction()
  
  def left_boundary(self):
    return self._impl.left_boundary()
  
  def right_boundary(self):
    return self._impl.right_boundary()
  
  def central_line(self):
    return self._impl.central_line()

  def reference_line(self):
    return self._impl.reference_line()
  
  def length(self):
    return self._impl.length() 
  
  def lane_section(self):
    road_impl = impl_cache.get_impl_by_ids("road", self._impl.road())
    if len(road_impl) == 0:
      pass
    for lane_section_impl in road_impl.lane_sections():
      for lane_id in lane_section_impl.lanes():
        if lane_id == self.id():
          return ToInfo(lane_section_impl, "lane_section")[0]
    return None
  
  def road(self):
    road_impl = impl_cache.get_impl_by_ids("road", self._impl.road())
    if len(road_impl) == 0:
      pass
    return ToInfo(road_impl, "road")[0]
    
  def successor_lane_links(self):
    return ToInfo(self._impl.successor_lane_links(), "lane_link") 
  
  def predecessor_lane_links(self):
    return ToInfo(self._impl.predecessor_lane_links(), "lane_link") 
  
  def left_neighbor_forward_lane(self):
    impls = impl_cache.get_impl_by_ids("lane", self._impl.left_neighbor_forward_lane())
    return ToInfo(impls, "lane")[0]
  
  def left_neighbor_forward_driving_lane(self):
    impls = impl_cache.get_impl_by_ids("lane", self._impl.left_neighbor_forward_driving_lane())
    return ToInfo(impls, "lane")[0]
  
  def right_neighbor_forward_lane(self):
    impls = impl_cache.get_impl_by_ids("lane", self._impl.right_neighbor_forward_lane())
    return ToInfo(impls, "lane")[0]
  
  def right_neighbor_forward_driving_lane(self):
    impls = impl_cache.get_impl_by_ids("lane", self._impl.right_neighbor_forward_driving_lane())
    return ToInfo(impls, "lane")[0]
  
  def left_neighbor_reverse_lane_ids(self):
    impls = impl_cache.get_impl_by_ids("lane", self._impl.left_neighbor_reverse_lane_ids())
    return ToInfo(impls, "lane")
  
  def right_neighbor_reverse_lane_ids(self):
    impls = impl_cache.get_impl_by_ids("lane", self._impl.right_neighbor_reverse_lane_ids())
    return ToInfo(impls, "lane")
  
  def lanes(self):
    return self._impl.lanes()
  
  def stop_lines(self):
    return self._impl.stop_lines()
  
  def cross_walks(self):
    return self._impl.cross_walks()
  
  def pois(self):
    return self._impl.pois()
  
  def pull_over_regions(self):
    return self._impl.pull_over_regions()
  
  def parking_spaces(self):
    return self._impl.parking_spaces()
  
  def objects(self):
    return self._impl.objects()
  
  def junctions(self):
    return self._impl.junctions()  
  
  def GetProjection(self, point:Vec2d):
    return self._impl.GetProjection(point)
  
  def GetPoint(self, s, d):
    return self._impl.GetProjection(s, d)
  
  def GetHeading(self, s):
    return self._impl.GetProjection(s)
  
  def GetWidth(self, s):
    return self._impl.GetProjection(s)
  
  def GetRoadWidth(self, s):
    return self._impl.GetProjection(s)
  
  def GetRelativeWidth(self, point:Vec2d, heading):
    return self._impl.GetProjection(point, heading)
  
class LanelinkInfo:
  def __init__(self, lane_link_impl:LaneLinkImpl) -> None:
    self._impl = lane_link_impl
    
  def from_lane(self):
    lane_impl = impl_cache.get_impl_by_ids("lane", self._impl.from_lane_id())
    return ToInfo(lane_impl, "lane")
  
  def from_s(self):
    return self._impl.from_s()
  
  def to_lane(self):
    lane_impl = impl_cache.get_impl_by_ids("lane", self._impl.to_lane_id())
    return ToInfo(lane_impl, "lane")
  
  def to_s(self):
    return self._impl.to_s()
  
class LaneSectionInfo:
  def __init__(self, lane_section_impl:LaneSectionImpl) -> None:
    self._impl = lane_section_impl
    
  def lanes(self):
    lane_impls = impl_cache.get_impl_by_ids("lane", self._impl.lanes())
    return ToInfo(lane_impls, "lane")
  
  def road(self):
    road_impl = impl_cache.get_impl_by_ids("road", self._impl.road_id())[0]
    return ToInfo([road_impl], "road")[0]
  
  def prev_lane_section(self):
    road_impl = impl_cache.get_impl_by_ids("road", self._impl.road_id())[0]
    for lane_section_impl in road_impl.lane_sections():
      if lane_section_impl.index == self._impl.index() - 1:
        return ToInfo(lane_section_impl, "lane_section")
    return None
  
  def next_lane_section(self):
    road_impl = impl_cache.get_impl_by_ids("road", self._impl.road_id())[0]
    for lane_section_impl in road_impl.lane_sections():
      if lane_section_impl.index == self._impl.index() + 1:
        return ToInfo(lane_section_impl, "lane_section")
    return None
       
   
class RoadInfo:
  def __init__(self, road_impl:RoadImpl) -> None:
    self._impl = road_impl
    self._lane_section_infos = ToInfo(road_impl.lane_sections())
    
  def id(self):
    return self._impl.id()
  
  def lane_sections(self):
    return self._lane_section_infos
  
  def junction_id(self):
    return self._impl.junction()
  
  def junction(self):
    junction_impl = impl_cache.get_impl_by_ids("junction", self._impl.junction())[0]
    return ToInfo(junction_impl, "junction")
  
  def IsInJunction(self):
    return self._impl.IsInJunction()
  
  def prev_roads(self):
    road_impls = impl_cache.get_impl_by_ids("road", self._impl.prev_roads())
    return ToInfo(road_impls, "road")
  
  def next_roads(self):
    road_impls = impl_cache.get_impl_by_ids("road", self._impl.next_roads())
    return ToInfo(road_impls, "road")
  
  def prev_junction(self):
    junction_impl = impl_cache.get_impl_by_ids("junction", self._impl.prev_junction())
    return ToInfo(junction_impl, "junction")
  
  def next_junction(self):
    junction_impl = impl_cache.get_impl_by_ids("junction", self._impl.next_junction())
    return ToInfo(junction_impl, "junction")
  
  def polygon(self):
    return self._impl.polygon()
  
  def attributes(self):
    return self._impl.attributes()
  
  def name(self):
    return self._impl.name()
  
  def type(self):
    return self._impl.type()
  
  # road struct and schedule poi todo
  
class JunctionInfo:
  def __init__(self, junction_impl:JunctionImpl) -> None:
    self._impl = junction_impl
    
  def id(self):
    return self._impl.id()
  
  def polygon(self):
    return self._impl.polygon()
  
  def type(self):
    return self._impl.type()
  
  def name(self):
    return self._impl.name()
  
  def is_virtual_junction(self):
    return self._impl.is_virtual_junction()
  
  def roads(self):
    road_impls = impl_cache.get_impl_by_ids("road", self._impl.roads())
    return ToInfo(road_impls, "road")
  
  def attributes(self):
    return self._impl.attributes()
  
  def lanes(self):
    return self._impl.lanes()
    
class ObjectInfo:
  def __init__(self, object_impl:ObjectImpl) -> None:
    self._impl = object_impl
  
  def id(self):
    return self._impl.id()    
  
  def type(self):
    return self._impl.type()
  
  def polygon(self):
    return self._impl.polygon()
  
  def height(self):
    return self._impl.height()
  
  def lanes(self):
    return self._impl.lanes()
  
  def attributes(self):
    return self._impl.attibutes()
  
class CrosswalkInfo:
  def __init__(self, crosswalk_impl:CrosswalkImpl) -> None:
    self._impl = crosswalk_impl
    
  def id(self):
    return self._impl.id()
  
  def left_boundary(self):
    return self._impl.left_boundary()
  
  def right_boundary(self):
    return self._impl.right_boundary()
  
  def polygon(self):
    return self._impl.polygon()
  
  def attributes(self):
    return self._impl.attributes()
  
  def lanes(self):
    return self._impl.lanes()
  
class ParkingSpaceInfo:
  def __init__(self, parking_space_impl:ParkingSpaceImpl) -> None:
    self._impl = parking_space_impl
    
  def id(self):
    return self._impl.id()
  
  def left_boundary(self):
    return self._impl.left_boundary()
  
  def right_boundary(self):
    return self._impl.right_boundary()
  
  def reference_line(self):
    return self._impl.reference_line()
  
  def polygon(self):
    return self._impl.polygon()
  
  def in_lanes(self):
    return self._impl.in_lanes()
  
  def out_lanes(self):
    return self._impl.out_lanes()
  
  def pois(self):
    return self._impl.pois()
  
class PoiInfo:
  def __init__(self, poi_impl:PoiImpl) -> None:
    self._impl = poi_impl
    
  def id(self):
    return self._impl.id()
  
  def type(self):
    return self._impl.type()
  
  def name(self):
    return self._impl.name()
  
  def coordinate(self):
    return self._impl.coordinate()
  
  def lanes(self):
    return self._impl.lanes()
  
  def pull_over_regions(self):
    return self._impl.pull_over_regions()
  
  def parking_spaces(self):
    return self._impl.parking_spaces()
  
class PullOverRegionInfo:
  def __init__(self, pull_over_region_impl:PullOverRegionImpl) -> None:
    self._impl = pull_over_region_impl
    
  def id(self):
    return self._impl.id()
  
  def pois(self):
    return self._impl.pois()
  
  def lane(self):
    lane_impl = impl_cache.get_impl_by_ids("lane", self._impl.lane())
    return ToInfo(lane_impl, "lane")
  
  def start_s(self):
    return self._impl.start_s()
  
  def end_s(self):
    return self._impl.end_s()
  
  def d(self):
    return self._impl.d()
  
class SignalInfo:
  def __init__(self, signal_impl:SignalImpl) -> None:
    self._impl = signal_impl
    
  def id(self):
    return self._impl.id()
  
  def polygon(self):
    return self._impl.polygon()
  
  def stop_lines(self):
    return self._impl.stop_lines()
  
  def signal_type(self):
    return self._impl.signal_type()
  
  def combination_type(self):
    return self._impl.combination_type()
  
  def signal_center(self):
    return self._impl.signal_center()
  
class StopLineInfo:
  def __init__(self, stop_line_impl:StopLineImpl) -> None:
    self._impl = stop_line_impl
    
  def id(self):
    return self._impl.id()
  
  def curve(self):
    return self.curve()
  
  def lanes(self):
    return self.lanes()
  
  def signals(self):
    return self.signals()
  
  
  
    
    
  


from threading import RLock
single_lock = RLock()

def Singleton(cls):
    instance = {}

    def _singleton_wrapper(*args, **kargs):
        with single_lock:
            if cls not in instance:
                instance[cls] = cls(*args, **kargs)
        return instance[cls]

    return _singleton_wrapper


# -*- coding: utf-8 -*-
from __future__ import absolute_import
import sys
if sys.version[0] == "2":
    from .py2.lib_hdmap_pyutils import *
else:
    from .py3.lib_hdmap_pyutils import *

__all__ = [
    # common
    "SetRuntimeFlags", "GetRuntimeFlags", "InitLogging",
    # math
    "Vec2d", "SDPoint", "LineSegment2d", "AABox2d", "Box2d", "Polygon2d", "Curve", "CloudServiceVessel", 
    "SetRuntimeFlags", "GetRuntimeFlags",
    "HDMapManager", "HDMap", "MapInfo", "HdmapVessel", "BlockLane", "BridgeMode", 
    "IdInfo", "LaneBoundaryTypeWithRange", "AttributesInfo", "LineStringDataInfo", "LineStringInfo",
    "CurveInfo", "LaneLinkInfo", "LaneInfo", "LaneSectionInfo", "RoadInfo", "JunctionInfo", "CrosswalkInfo",
    "StopLineInfo", "SignalInfo", "PullOverRegionInfo", "ParkingSpaceInfo", "PoiInfo", "RoadStructInfo", "LaneIndexInfo",
    "ObjectInfo", "RelationInfo", "Range", "ReferencePoint", "ReferenceLine", "OneExtendConfig", "BezierConfig", "Utils"
]



from proto.hdmap_lib.proto import object_pb2
from externals.libhdmap import *
from pyhdmap.remote.hdmap_impl.relation_impl import RelationImpl
from pyhdmap.remote.hdmap_impl.attribute_impl import AttributesImpl

class ObjectImpl:
  def __init__(self) -> None:
    self._id = 0
    self._type = object_pb2.Object.Type.NONE
    self._polygon = Polygon2d()
    self._height = 0
    self._lanes:list[RelationImpl] = [] 
    self._attributes = AttributesImpl()
  
  def id(self):
    return self._id
  
  def type(self):
    return self._type
  
  def polygon(self):
    return self._polygon
  
  def height(self):
    return self._height
  
  def lanes(self):
    return self._lanes
  
  def attibutes(self):
    return self._attributes